//
//  CSController.mm
//  Chimera
//
//  Created by Simon Woodside on Fri Feb 28 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "CSController.h"

#import "BrowserWrapper.h"
#import "CSDataSource.h"
//#import "CSFormatter.h"

@implementation CSController

- (id)init;
{
  NSLog(@"(CSController) -init");
  if( (self = [super init]) )
  {
    dataSource = [[[CSDataSource alloc] init] retain];
    viewDict = [[NSMutableDictionary dictionaryWithCapacity:10] retain];
    delegate = dataSource; // just a shim
  }
  return self;
}

- (void)setNeedsUpdate;
{
  NSLog(@"(CSController) -setNeedsUpdate");
  // Just do it now
  NSEnumerator * enumerator = [viewDict objectEnumerator];
  NSScrollView * anObject;
  while( (anObject = (NSScrollView *)[enumerator nextObject]) ) {
    [(NSOutlineView *)[anObject documentView] reloadItem:[dataSource rootNode]
                                          reloadChildren:YES];
  }
}

// Called by someone who just loaded my view nib
- (void)didOpenWindow:(BrowserWindowController *)aWindow;
{
  //scrollView should be automagically updated to the new one from the nib
  if( scrollView )
  {
    [viewDict setObject:[scrollView retain] forKey:aWindow];
    //configure the new outline view
    NSOutlineView * outlineView = [scrollView documentView];
    [[outlineView tableColumnWithIdentifier:@"icon"] setDataCell:
      [[NSImageCell alloc] initImageCell:nil]];
    [outlineView expandItem:[dataSource rootNode]];
  } else {
  }
}
- (void)willCloseWindow:(BrowserWindowController *)aWindow;
{
  //the BrowserViews should already be taken care of
  [viewDict removeObjectForKey:aWindow];
  [self setNeedsUpdate];
}

- (NSView *)viewForWindow:(BrowserWindowController *)bwc;
{
  [viewDict objectForKey:bwc];
}

- (void)addCSWithBrowserWrapper:(BrowserWrapper *)bw;
{
  [bw addListener:self]; // let's be a listener
  [dataSource addConcurrentSession:bw];
  [self setNeedsUpdate];
}
- (void)removeCSWithBrowserWrapper:(BrowserWrapper *)bw;
{
  [bw removeListener:self];
  [dataSource removeConcurrentSession:bw];
  [self setNeedsUpdate];
}

#pragma mark -
#pragma mark nsBrowserListener delegates

// Sort-of-delegates
// since this ns listener gives us no idea who changed,
// we'll just update everyone
- (void)onLoadingStarted;
{
  [self setNeedsUpdate];
}
- (void)onLoadingCompleted:(BOOL)succeeded;
{
  [self setNeedsUpdate];
}
- (void)onLocationChange:(NSString*)urlSpec;
{
  [self setNeedsUpdate];
}
- (void)onProgressChange:(int)currentBytes outOf:(int)maxBytes;
{ return; }
- (void)onStatusChange:(NSString*)aMessage;
{ return; }
- (void)onSecurityStateChange:(unsigned long)newState;
{ return; }
- (void)onShowTooltip:(NSPoint)where withText:(NSString*)text;
{ return; }
- (void)onHideTooltip;
{ return; }

#pragma mark -
#pragma mark NSOutlineViewDataSource protocol
  //NSOutlineViewDataSource informal protocol methods
- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item;
{ return [delegate outlineView:outlineView child:index ofItem:item]; }
- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item;
{ return [delegate outlineView:outlineView isItemExpandable:item]; }
- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item;
{ return [delegate outlineView:outlineView numberOfChildrenOfItem:item]; }
- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item;
{ return [delegate outlineView:outlineView objectValueForTableColumn:tableColumn byItem:item]; }
  //NSOutlineView informal protocol methods
- (void)outlineViewSelectionDidChange:(NSNotification *)notification;
{ return [delegate outlineViewSelectionDidChange:notification]; }

@end
