//
//  CSDataSource.mm
//  Chimera
//
//  Created by Simon Woodside on Fri Feb 28 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "CSDataSource.h"

#import "BrowserWrapper.h"
#import "TruncatingTableViewString.h"

@implementation CSDataSource

- (id)init;
{
  if( (self = [super init]) ) {
    NSLog(@"SIMON (CSDataSource) init");
    csArray = [[NSMutableArray arrayWithCapacity:10] retain];
    rootString = @"Sessions";
  }
  return self;
}

#pragma mark -

- (void)addConcurrentSession:(BrowserWrapper *)bw;
{
  [csArray addObject:bw];
  NSLog(@"SIMON (CSDataSource) addConcurrentSession count = %d", [csArray count]);
}

- (void)removeConcurrentSession:(BrowserWrapper *)bw;
{
  [csArray removeObject:bw];
}

- (NSString *)rootNode;
{
  return rootString;
}

- (int)columnIndexForItem:(id)item;
{
  //name column
  if( item == rootString ) {
    return 0;
  }
  if( item && [item isKindOfClass:[BrowserWrapper class]] ) {
    return [csArray indexOfObject:item] +1;
  }
  //shouldn't happen I don't think
  return 0;
}

#pragma mark -
#pragma mark NSOutlineViewDataSource protocol

//
//NSOutlineViewDataSource informal protocol methods
//

//child ofItem delegate
- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item;
{
  if( item == nil ) {
    //always the root
    return rootString;
  }
  if( item == rootString && index < (int)[csArray count] ) {
    //valid child (translated later)
    return [csArray objectAtIndex:index];
  }
  //nothing there yet, or grandchild
  return nil;
}

//isItemExpandable delegate
- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item;
{
  if( item == rootString ) {
    //only rootString is expandable
    return YES;
  }
  return NO;
}

//numberOfChildrenOfItem delegate
- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item;
{
  if( item == nil ) {
    //rootString
    return 1;
  } else if( item == rootString && [csArray count] ) {
    //the children
    return [csArray count];
  }
  //no grandchildren or whatever
  return 0;
}

//objectValueForTableColumn byItem delegate
- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item;
{
  if( ![[tableColumn identifier] isEqualToString:@"icon"] ) {
    //name column
    if( item == rootString ) {
      return rootString;
    }
    if( item && [item isKindOfClass:[BrowserWrapper class]] ) {
      NSString * resultString = [(BrowserWrapper *) item title];
      resultString =
        [resultString middleTruncatedStringWithIndicatorString:@"..."
                                            fittingTableColumn:tableColumn];
      return resultString;
    }
    //shouldn't happen I don't think
    return @"WTF???";
  } else {
    //icon column
    if( item && [item isKindOfClass:[BrowserWrapper class]] ) {
      return [(BrowserWrapper *)item siteIconImage];
    } else {
      return [NSImage imageNamed:@"globe_ico"];
    }
  }
}

#pragma mark -
#pragma mark NSOutlineView protocol

//NSOutlineView informal protocol methods
//
//
- (void)outlineViewSelectionDidChange:(NSNotification *)notification;
{
  //The notification object is the NSOutlineView whose selection changed
  NSLog(@"(CSDataSource) outlineViewSelectionDidChange");
  NSOutlineView * aView = (NSOutlineView *)[notification object];
  NSTabViewItem * tabViewItem =
    [(BrowserWrapper *)[aView itemAtRow:[aView selectedRow]] tab];
  //select the right tab
  [[tabViewItem tabView] selectTabViewItem:tabViewItem ];
  //make window key and bring it to front in case it aint
  [[[tabViewItem tabView] window] makeKeyAndOrderFront:self];
}

@end
